@@ -53,9 +53,13 @@ class DatabaseInvalidFilterError(_DatabaseInvalidError):
     type = "filter"
 
 
-
 class Database:
-    required_version = "0.9.1"
+    required_version = "0.9.11"
+    
+    # We reference preludedb_sql_destroy since it might be deleted
+    # prior Database.__del__() is called.
+    _sql_destroy = preludedb_sql_destroy
+    _sql = None
     
     def __init__(self, config):
         settings = preludedb_sql_settings_new()
@@ -78,7 +82,7 @@ class Database:
 
         # check if the database has been created
         try:
-            version = self.query("SELECT * FROM Prewikka_Version")[0][0]
+            version = self.query("SELECT version FROM Prewikka_Version")[0][0]
         except PreludeDBError, e:
             print >> sys.stderr, e
             print >> sys.stderr, "The Prewikka database does not seem to have been created."
@@ -89,7 +93,8 @@ class Database:
             sys.exit(1)
 
     def __del__(self):
-        preludedb_sql_destroy(self._sql)
+        if self._sql:
+            self._sql_destroy(self._sql)
         
     def queries_from_file(self, filename):
         content = open(filename).read()
@@ -143,7 +148,7 @@ class Database:
         return "'" + utils.time_to_ymdhms(time.localtime(t)) + "'"
 
     def hasUser(self, login):
-        rows = self.query("SELECT login FROM Prewikka_User WHERE login = '%s'" % login)
+        rows = self.query("SELECT login FROM Prewikka_User WHERE login = %s" % self.escape(login))
         
         return bool(rows)
 
@@ -157,7 +162,7 @@ class Database:
         self.query("DELETE FROM Prewikka_User WHERE login = %s" % login)
         self.query("DELETE FROM Prewikka_Permission WHERE login = %s" % login)
         self.query("DELETE FROM Prewikka_Session WHERE login = %s" % login)
-        self.query("DELETE FROM Prewikka_Filter_Criterion USING Prewikka_Filter "
+        self.query("DELETE FROM Prewikka_Filter_Criterion USING Prewikka_Filter, Prewikka_Filter_Criterion "
                    "WHERE Prewikka_Filter.login = %s AND Prewikka_Filter.id = Prewikka_Filter_Criterion.id" % login)
         self.query("DELETE FROM Prewikka_Filter WHERE login = %s" % login)
         self.transaction_end()
@@ -186,7 +191,7 @@ class Database:
         return map(lambda r: r[0], self.query("SELECT login FROM Prewikka_User"))
         
     def getUser(self, login):
-        return User.User(self, login, self.getPermissions(login), self.getConfiguration(login))
+        return User.User(self, login, self.getLanguage(login), self.getPermissions(login), self.getConfiguration(login))
 
     def setPassword(self, login, password):
         self.query("UPDATE Prewikka_User SET password=%s WHERE login = %s" % (self.escape(password), self.escape(login)))
@@ -201,6 +206,16 @@ class Database:
     def hasPassword(self, login):
         return bool(self.query("SELECT password FROM Prewikka_User WHERE login = %s AND password IS NOT NULL" % self.escape(login)))
 
+    def setLanguage(self, login, lang):
+        self.query("UPDATE Prewikka_User SET lang=%s WHERE login = %s" % (self.escape(lang), self.escape(login)))
+        
+    def getLanguage(self, login):
+        rows = self.query("SELECT lang FROM Prewikka_User WHERE login = %s" % (self.escape(login)))
+        if len(rows) > 0:
+            return rows[0][0]
+        
+        return None
+        
     def setPermissions(self, login, permissions):
         self.transaction_start()
         self.query("DELETE FROM Prewikka_Permission WHERE login = %s" % self.escape(login))
@@ -255,6 +270,9 @@ class Database:
     def getAlertFilter(self, login, name):
         rows = self.query("SELECT id, comment, formula FROM Prewikka_Filter WHERE login = %s AND name = %s" %
                           (self.escape(login), self.escape(name)))
+        if len(rows) == 0:
+            return None
+            
         id, comment, formula = rows[0]
         elements = { }
         for element_name, path, operator, value in \
@@ -267,3 +285,5 @@ class Database:
         id = long(self.query("SELECT id FROM Prewikka_Filter WHERE login = %s AND name = %s" % (self.escape(login), self.escape(name)))[0][0])
         self.query("DELETE FROM Prewikka_Filter WHERE id = %d" % id)
         self.query("DELETE FROM Prewikka_Filter_Criterion WHERE id = %d" % id)
+
+
